###############   PSQF 6249 FA & SEM Example 6a Ordinal IFA using R   #############

# Set width of output and  number of digits to be printed
options("width"=140, "digits"=4) 

# Use this to turn off scientific notation: 
# "fixed notation will be preferred unless it is more than 'scipen' digits wide"
options(scipen=9)

# turn off stars for different alpha levels
options(show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("readxl")) install.packages("readxl")
library(readxl) # To import xls or xlsx data as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sort

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

if (!require("psych")) install.packages("psych")
library(psych) # To add summary functions

if (!require("lavaan")) install.packages("lavaan")
library(lavaan) # To fit SEMs

# Other packages not used here, but which are pry useful

#if (!require("semTools")) install.packages("semTools")
#library(semTools) # lavaan add-ons (multiple imputation, moderation)

#if (!require("MplusAutomation")) install.packages("MplusAutomation")
#library(lavaan) # To use Mplus within R

#######################################################################################

# Define variables for working directory and data name
filesave = "C:\\Dropbox/22_PSQF6249/OCTO/IRT Single-Group/PSQF6249_Example6a/"
filename = "Example6a.xlsx"
setwd(dir=filesave)

# Import OCTO item-level data
Example6a = read_excel(paste0(filesave,filename)) 
# Convert to data frame as data frame to use for analysis
Example6a = as.data.frame(Example6a)

# Making a list of variables to be used across functions below
ordinalVars = c("cia1","cia2","cia3","cia4","cia5","cia6","cia7")

# Open external file to save results to
txtStart(file=paste0(filesave,"R_Example6a_Ordinal_IFA_Output.txt"))


########################################################################################

# Polychoric Correlations using lavCor from lavaan: Pairwise Correlation Matrix
lavCor(Example6a, ordered=ordinalVars, missing="pairwise", se="none", output="cor")

txtComment(" ") # insert blank space in output

# Polychoric Correlations using lavCor from lavaan: Saturated Model Results
lavCor(Example6a, ordered=ordinalVars, missing="pairwise", se="standard", output="est")

txtComment(" ") # insert blank space in output

# Regular Pearson Correlations using lavCor from lavaan: Pairwise Correlation Matrix
lavCor(Example6a, ordered=NULL, missing="pairwise", se="none", output="cor")

txtComment(" ") # insert blank space in output

# Regular Pearson Correlations using lavCor from lavaan: Saturated Model Results
lavCor(Example6a, ordered=NULL, missing="pairwise", se="standard", output="est")

txtComment(" ") # insert blank space in output

#### General notes about lavaan syntax objects:
# Once factors are defined they can be used in syntax as if they were observed variables
# Multiple commands can go on a single line separated by ; 
# Parameters can be fixed to constant values by using the * command
# Start values can be specified by using value* in front of parameter 
# Labels can be implemented with (label)* in front of parameter

# Example of how to concatenate syntax strings into one model
#Model <- sem(model=c(syntax1, syntax2), data = myData)

########################################################################################

# Two-Parameter Model: Single Factor Using Fully Z-Scored Factor Scaling
# Factor Variance = 1, Factor Mean = 0, so all item loadings and thresholds estimated

# R Syntax for lavaan function: longer but more transparent version of model
Syntax2P = "
# Define factor and request item factor loadings --> factor =~ item + item + item
  IADL =~ cia1 + cia2 + cia3 + cia4 + cia5 + cia6 + cia7
# Item thresholds (=intercept*-1) use the | operator and start at value 1 after t
  cia1 | t1; cia2 | t1; cia3 | t1; cia4 | t1; cia5 | t1; cia6 | t1; cia7 | t1 
  cia1 | t2; cia2 | t2; cia3 | t2; cia4 | t2; cia5 | t2; cia6 | t2; cia7 | t2 
  cia1 | t3; cia2 | t3; cia3 | t3; cia4 | t3; cia5 | t3; cia6 | t3; cia7 | t3 
# Item error variances fixed=1 for identification
  cia1 ~~ 1*cia1; cia2 ~~ 1*cia2; cia3 ~~ 1*cia3; cia4 ~~ 1*cia4
  cia5 ~~ 1*cia5; cia6 ~~ 1*cia6; cia7 ~~ 1*cia7
# Factor variance fixed=1 and factor mean fixed=0 (* means fixed in lavaan)
  IADL ~~ 1*IADL; IADL ~ 0
"
# Use WLSMV estimation like in Mplus, z-scored latent variables (mean=0, SD=1)
  Model2P = lavaan(model=Syntax2P, data=Example6a, mimic="mplus", std.lv=TRUE,  
                   estimator="WLSMV", parameterization="theta", ordered=ordinalVars,
                   missing="pairwise")
# Print solution: get fit, get effect size, STDYX solution
  summary(object=Model2P, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
# Get model-implied means, variances, and covariances
  fitted(object=Model2P)
# Get residual (discrepancy) info for local misfit in covariance and correlation form
  resid(object=Model2P, type="raw"); resid(object=Model2P, type="cor")
# Get normalized residuals (like z-scores for discrepancies)
  resid(object=Model2P, type="normalized")
# Get "cheat code" modification indices
  modificationindices(object=Model2P, sort.=TRUE)

txtComment(" ") # insert blank space in output

########################################################################################
# One-Parameter Model: Single Factor Using Fully Z-Scored Factor Scaling
# Factor Variance = 1, Factor Mean = 0, all thresholds estimated, all loadings EQUAL

# R Syntax for lavaan function: longer but more transparent version of model
Syntax1P = "
# Define factor and request item factor loadings --> factor =~ item + item + item
  IADL =~ (L)*cia1 + (L)*cia2 + (L)*cia3 + (L)*cia4 + (L)*cia5 + (L)*cia6 + (L)*cia7
# Item thresholds (=intercept*-1) use the | operator and start at value 1 after t
  cia1 | t1; cia2 | t1; cia3 | t1; cia4 | t1; cia5 | t1; cia6 | t1; cia7 | t1 
  cia1 | t2; cia2 | t2; cia3 | t2; cia4 | t2; cia5 | t2; cia6 | t2; cia7 | t2 
  cia1 | t3; cia2 | t3; cia3 | t3; cia4 | t3; cia5 | t3; cia6 | t3; cia7 | t3 
# Item error variances fixed=1 for identification
  cia1 ~~ 1*cia1; cia2 ~~ 1*cia2; cia3 ~~ 1*cia3; cia4 ~~ 1*cia4
  cia5 ~~ 1*cia5; cia6 ~~ 1*cia6; cia7 ~~ 1*cia7
# Factor variance fixed=1 and factor mean fixed=0 (* means fixed in lavaan)
  IADL ~~ 1*IADL; IADL ~ 0
"
# Use WLSMV estimation like in Mplus, z-scored latent variables (mean=0, SD=1)
Model1P = lavaan(model=Syntax1P, data=Example6a, mimic="mplus", std.lv=TRUE,  
                 estimator="WLSMV", parameterization="theta", ordered=ordinalVars,
                 missing="pairwise") 
# Print solution: get fit, get effect size, STDYX solution
summary(object=Model1P, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
# Get model-implied means, variances, and covariances
fitted(object=Model1P)
# Get residual (discrepancy) info for local misfit in covariance and correlation form
resid(object=Model1P, type="raw"); resid(object=Model1P, type="cor")
# Get normalized residuals (like z-scores for discrepancies)
resid(object=Model1P, type="normalized")
# Get "cheat code" modification indices
modificationindices(object=Model1P, sort.=TRUE)

txtComment(" ") # insert blank space in output

print("Approximate DIFFTEST model comparison from Mplus")
print(anova(Model2P,Model1P), digits=6)


# Close external file of saved results
txtStop()


